﻿using System;
using System.IO;
using System.Linq;
// using Microsoft.Extensions.Configuration;
using System.Web;
using GrapeCity.ActiveReports.Aspnet.Designer;
using GrapeCity.ActiveReports.Aspnet.Viewer;

using WebDesigner_MVC.Implementation;
using WebDesigner_MVC.Services;
using Owin;
using Microsoft.Owin;
using Microsoft.Owin.StaticFiles;
using Microsoft.Owin.FileSystems;
using System.Web.Mvc;
using System.Web.Routing;
using System.Reflection;
using GrapeCity.ActiveReports.Aspnet.Designer.Services;
using GrapeCity.ActiveReports.Aspnet.Designer.Utilities;
using System.Xml;
using System.Collections.Generic;
using GrapeCity.ActiveReports.PageReportModel;
using GrapeCity.ActiveReports;

[assembly: OwinStartup(typeof(WebDesigner_MVC.Startup))]

namespace WebDesigner_MVC
{
	public class Startup
	{
		private static readonly DirectoryInfo ResourcesRootDirectory =
			new DirectoryInfo(String.Format(@"{0}.\resources\", HttpRuntime.AppDomainAppPath));
		private static readonly DirectoryInfo TemplatesRootDirectory =
			new DirectoryInfo(String.Format(@"{0}.\templates\", HttpRuntime.AppDomainAppPath));
		private static readonly DirectoryInfo DataSetsRootDirectory =
			new DirectoryInfo(String.Format(@"{0}.\datasets\", HttpRuntime.AppDomainAppPath));

		public void Configuration(IAppBuilder app)
		{
			app.UseErrorPage();

			var dataSetsService = new FileSystemDataSets(DataSetsRootDirectory);
			var templatesService = new FileSystemTemplates(TemplatesRootDirectory);
            var resoruceService = new FileSystemResources(ResourcesRootDirectory);
            app.Use((context, next) =>
			{
				context.Set(typeof(IDataSetsService).ToString(), dataSetsService);
				context.Set(typeof(ITemplatesService).ToString(), templatesService);
              //  context.Set(typeof(GrapeCity.ActiveReports.Aspnet.Designer.Services.IResourcesService).ToString(), resoruceService);
				return next.Invoke();
			});
           // app.UseDesigner(config => config.UseFileStore(ResourcesRootDirectory));
            // app.UseStaticFiles(ResourcesRootDirectory.FullName);
           // app.UseReporting(config => { config.UseFileStore(ResourcesRootDirectory); });
            	app.UseDesigner(config => config.UseCustomStore(resoruceService));
            // app.UseStaticFiles(ResourcesRootDirectory.FullName);
            app.UseReporting(config => {  config.UseFileStore(ResourcesRootDirectory);});

            app.UseStaticFiles(new StaticFileOptions { FileSystem = new PhysicalFileSystem(String.Format(@"{0}.\wwwroot\", HttpRuntime.AppDomainAppPath)) });
		}
	}
    public class FileSystemResources : IResourcesService
    {
        private readonly Func<string, object> _getReportFn;
        private readonly DirectoryInfo _rootFolder;
        List<ThemeInfo> themesInfo = new List<ThemeInfo>();
        List<ImageInfo> imagesInfo = new List<ImageInfo>();
        List<ReportInfo> reportInfo = new List<ReportInfo>();
        public object GetReport(string reportId) => _getReportFn(_rootFolder.FullName + @"\" + reportId);
        public FileSystemResources(DirectoryInfo rootFolder)
        {
            _rootFolder = rootFolder;
        }
        public void DeleteReport(string id)
        {
            if (string.IsNullOrEmpty(id)) throw new ArgumentNullException(nameof(id));

            var reportName = GetResourceName(id);

            var fullPath = Path.Combine(_rootFolder.FullName, reportName);
            if (File.Exists(fullPath)) File.Delete(fullPath);

            var dirPath = Path.GetDirectoryName(fullPath);
            if (dirPath != _rootFolder.FullName && !Directory.GetFiles(dirPath).Any()) Directory.Delete(dirPath);
        }

        public byte[] GetImage(string id, out string mimeType)
        {
            System.Drawing.Image img = System.Drawing.Image.FromFile(_rootFolder.FullName + @"\" + id);
            mimeType = "image/png";
            using (var ms = new MemoryStream())
            {
                img.Save(ms, img.RawFormat);
                return ms.ToArray();
            }

        }

        public IImageInfo[] GetImagesList()
        {
            imagesInfo.Clear();
            List<string> imagesfiles = Directory.GetFiles(_rootFolder.FullName, "*.*", SearchOption.AllDirectories)
       .Where(file => new string[] { ".jpg", ".png" }
       .Contains(Path.GetExtension(file)))
       .ToList();
            foreach (string t in imagesfiles)
            {
                imagesInfo.Add(new ImageInfo() { Id = Path.GetFileName(t), Name = Path.GetFileName(t) });

            }

            return imagesInfo.ToArray();
        }

        Report IResourcesService.GetReport(string id)
        {

            PageReport pr = new PageReport(new FileInfo(_rootFolder.FullName + @"\" + id));


            return pr.Report;
        }

        public IReportInfo[] GetReportsList()
        {
            reportInfo.Clear();
            List<string> reportsfiles = Directory.GetFiles(_rootFolder.FullName, "*.*", SearchOption.AllDirectories)
        .Where(file => new string[] { ".rdlx" }
        .Contains(Path.GetExtension(file)))
        .ToList();

            foreach (string t in reportsfiles)
            {
                reportInfo.Add(new ReportInfo() { Id = Path.GetFileName(t), Name = Path.GetFileName(t) });

            }

            return reportInfo.ToArray();
        }

        public Theme GetTheme(string id)
        {

            Theme theme = new Theme();
            ThemeColors colors = new ThemeColors();
            ThemeFonts fonts = new ThemeFonts();
            ThemeFont majorfont = new ThemeFont();
            ThemeFont minorfont = new ThemeFont();
            theme.Colors = colors;

            fonts.MajorFont = majorfont;
            fonts.MinorFont = minorfont;
            theme.Fonts = fonts;
            if (themesInfo.Exists(t => t.Id == id))
            {
                System.Diagnostics.Debug.WriteLine(id);
                ThemeInfo themeInfo = themesInfo.Find(t => t.Id == id);

                theme.Colors.Dark1 = themeInfo.Dark1;
                theme.Colors.Dark2 = themeInfo.Dark2;
                theme.Colors.Light1 = themeInfo.Light1;
                theme.Colors.Light2 = themeInfo.Light2;
                theme.Colors.Accent1 = themeInfo.Accent1;
                theme.Colors.Accent2 = themeInfo.Accent2;
                theme.Colors.Accent3 = themeInfo.Accent3;
                theme.Colors.Accent4 = themeInfo.Accent4;
                theme.Colors.Accent5 = themeInfo.Accent5;
                theme.Colors.Accent6 = themeInfo.Accent6;
                theme.Fonts.MajorFont.Family = themeInfo.MajorFontFamily;
                theme.Fonts.MajorFont.Size = themeInfo.MajorFontSize;
                theme.Fonts.MajorFont.Style = themeInfo.MajorFontStyle;
                theme.Fonts.MajorFont.Weight = themeInfo.MajorFontWeight;
                theme.Fonts.MinorFont.Family = themeInfo.MinorFontFamily;
                theme.Fonts.MinorFont.Size = themeInfo.MinorFontSize;
                theme.Fonts.MinorFont.Style = themeInfo.MinorFontStyle;
                theme.Fonts.MinorFont.Weight = themeInfo.MinorFontWeight;

            }


            return theme;
        }

        public IThemeInfo[] GetThemesList()
        {
            themesInfo.Clear();
            List<string> themesfiles = Directory.GetFiles(_rootFolder.FullName, "*.*", SearchOption.AllDirectories)
       .Where(file => new string[] { ".rdlx-theme" }
       .Contains(Path.GetExtension(file)))
       .ToList();
            foreach (string t in themesfiles)
            {
                XmlDocument xmldoc = new XmlDocument();
                xmldoc.Load(t);
                XmlNamespaceManager nsmgr = new XmlNamespaceManager(xmldoc.NameTable);
                nsmgr.AddNamespace("ns", "http://www.datadynamics.com/sidewinder/theme");
                string id = Path.GetFileName(t);
                string title = Path.GetFileNameWithoutExtension(t);

                string dark1 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Dark1", nsmgr).InnerText;
                string dark2 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Dark2", nsmgr).InnerText;
                string light1 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Light1", nsmgr).InnerText;
                string light2 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Light2", nsmgr).InnerText;
                string accent1 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Accent1", nsmgr).InnerText;
                string accent2 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Accent2", nsmgr).InnerText;
                string accent3 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Accent3", nsmgr).InnerText;
                string accent4 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Accent4", nsmgr).InnerText;
                string accent5 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Accent5", nsmgr).InnerText;
                string accent6 = xmldoc.SelectSingleNode("/ns:Theme/ns:Colors/ns:Accent6", nsmgr).InnerText;
                string majorFontFamily = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MajorFont/ns:Family", nsmgr).InnerText;
                string majorFontStyle = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MajorFont/ns:Style", nsmgr).InnerText;
                string majorFontSize = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MajorFont/ns:Size", nsmgr).InnerText;
                string majorFontWeight = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MajorFont/ns:Weight", nsmgr).InnerText;
                string minorFontFamily = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MinorFont/ns:Family", nsmgr).InnerText;
                string minorFontStyle = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MinorFont/ns:Style", nsmgr).InnerText;
                string minorFontSize = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MinorFont/ns:Size", nsmgr).InnerText;
                //   string minorFontWeight = xmldoc.SelectSingleNode("/ns:Theme/ns:Fonts/ns:MinorFont/ns:Weight", nsmgr).InnerText;
                //  info.Add(new ThemeInfo() { Id = id, Title = title, Dark1 = dark1, Dark2 = dark2, Light1 = light1, Light2 = light2, Accent1 = accent1, Accent2 = accent2, Accent3 = accent3, Accent4 = accent4, Accent5 = accent5, Accent6 = accent6, MajorFontFamily = majorFontFamily, MinorFontFamily = minorFontFamily });

                themesInfo.Add(new ThemeInfo() { Id = id, Title = title, Dark1 = dark1, Dark2 = dark2, Light1 = light1, Light2 = light2, Accent1 = accent1, Accent2 = accent2, Accent3 = accent3, Accent4 = accent4, Accent5 = accent5, Accent6 = accent6, MajorFontFamily = majorFontFamily, MajorFontSize = majorFontSize, MajorFontStyle = majorFontStyle, MajorFontWeight = majorFontWeight, MinorFontFamily = minorFontFamily, MinorFontSize = minorFontSize, MinorFontStyle = minorFontStyle, MinorFontWeight = "" });
            }


            return themesInfo.ToArray();
        }
        private string TempReportPrefix = "temp";

        private string GetResourceId(string resourceName) => resourceName.Replace("/", "%2F");

        private string GetResourceName(string resourceId) => resourceId.Replace("%2F", "/");

        private string GetReportResourceName(string name)
        {
            var reportResourceName = GetResourceName(name);
            return reportResourceName;
        }
        public string SaveReport(string name, Report report, bool isTemporary = false)
        {

            var reportName = isTemporary
                         ? $"{Guid.NewGuid().ToString("D")}.rdlx"
                         : GetReportResourceName(name);

            report.Name = name;

            if (isTemporary)
            {
                reportName = $"{TempReportPrefix}{reportName}";
            }

            var reportFullPath = Path.Combine(_rootFolder.FullName, reportName);

            var dirPath = Path.GetDirectoryName(reportFullPath);
            if (!Directory.Exists(dirPath))
            {
                Directory.CreateDirectory(dirPath);
            }

            byte[] reportXml = ReportConverter.ToXml(report);

            var absoluteReferences = ReportResources.GetReferences(reportXml);

            var reportFullUri = new Uri(reportFullPath);
            var referencesMap = new Dictionary<string, string>();
            foreach (var resourceAbsolutePath in absoluteReferences)
            {
                if (!IsValidPath(_rootFolder.FullName, resourceAbsolutePath)) continue;
                if (!TryUpdateRelativePath(_rootFolder.FullName, reportFullPath, resourceAbsolutePath, out string resourceRelativePath)) continue;

                referencesMap.Add(resourceAbsolutePath, resourceRelativePath);
            }
            var updReportXml = ReportResources.UpdateReferences(reportXml, referencesMap);

            File.WriteAllBytes(reportFullPath, updReportXml);

            var reportId = GetResourceId(reportName.Replace('\\', '/'));

            return reportId;
        }
        private static bool IsValidPath(string basePath, string path)
        {
            try
            {
                var fullPath = Path.Combine(basePath, path);
                return File.Exists(fullPath);
            }
            catch
            {
                return false;
            }
        }
        private static bool TryUpdateRelativePath(string currBasePath, string nextBasePath, string path, out string updatedPath)
        {
            try
            {
                updatedPath = UpdateRelativePath(currBasePath, nextBasePath, path);
                return true;
            }
            catch
            {
                updatedPath = null;
                return false;
            }

        }

        private static string UpdateRelativePath(string currBasePath, string nextBasePath, string path)
        {
            var currFullPath = Path.Combine(currBasePath, path);
            var currFullPathUri = new Uri(currFullPath);
            var nextBasePathUri = new Uri(nextBasePath);
            var nextPathUri = nextBasePathUri.MakeRelativeUri(currFullPathUri);
            var nextPath = Uri.UnescapeDataString(nextPathUri.ToString());
            return nextPath;
        }
        public string UpdateReport(string id, Report report)
        {
            var reportName = GetResourceName(id);

            return SaveReport(reportName, report, false);
        }
    }

    class ReportInfo : IReportInfo
    {
        public string Id { get; set; }
        public string Name { get; set; }
        public string Type { get; set; }
    }

    class ImageInfo : IImageInfo
    {
        public string Id { get; set; }
        public string Name { get; set; }
        public byte[] Thumbnail { get; set; }
        public string ContentType { get; set; }
    }
    class ThemeInfo : IThemeInfo
    {
        public string Id { get; set; }
        public string Title { get; set; }
        public string Dark1 { get; set; }
        public string Dark2 { get; set; }
        public string Light1 { get; set; }
        public string Light2 { get; set; }
        public string Accent1 { get; set; }
        public string Accent2 { get; set; }
        public string Accent3 { get; set; }
        public string Accent4 { get; set; }
        public string Accent5 { get; set; }
        public string Accent6 { get; set; }
        public string MajorFontFamily { get; set; }
        public string MinorFontFamily { get; set; }
        public string MajorFontStyle { get; set; }
        public string MinorFontStyle { get; set; }
        public string MajorFontSize { get; set; }
        public string MinorFontSize { get; set; }
        public string MajorFontWeight { get; set; }
        public string MinorFontWeight { get; set; }
    }

}
